@@ -13,6 +13,7 @@ import (
 
 	"github.com/coreos/go-semver/semver"
 	route1client "github.com/openshift/client-go/route/clientset/versioned"
+	v1 "k8s.io/api/core/v1"
 	networkingv1 "k8s.io/api/networking/v1"
 	storagev1 "k8s.io/api/storage/v1"
 	apiequality "k8s.io/apimachinery/pkg/api/equality"
@@ -292,7 +293,7 @@ var _ = Describe("CDI ingress config tests, using manifests", func() {
 
 	It("[test_id:4950]Keep current value on no http rule ingress", func() {
 		manifestFile = "manifests/ingressNoHttp.yaml"
-		controllerPod, err := utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, cdiDeploymentPodPrefix, "app=containerized-data-importer")
+		controllerPod, err := utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, cdiDeploymentPodPrefix, common.CDILabelSelector)
 		Expect(err).ToNot(HaveOccurred())
 		currentRestarts := controllerPod.Status.ContainerStatuses[0].RestartCount
 		fmt.Fprintf(GinkgoWriter, "INFO: Current number of restarts: %d\n", currentRestarts)
@@ -310,7 +311,7 @@ var _ = Describe("CDI ingress config tests, using manifests", func() {
 		for i := 0; i < 10; i++ {
 			// Check for 20 seconds if the deployment pod crashed.
 			time.Sleep(2 * time.Second)
-			controllerPod, err = utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, cdiDeploymentPodPrefix, "app=containerized-data-importer")
+			controllerPod, err = utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, cdiDeploymentPodPrefix, common.CDILabelSelector)
 			Expect(err).ToNot(HaveOccurred())
 			// Restarts will increase if we crashed due to null pointer.
 			Expect(currentRestarts).To(Equal(controllerPod.Status.ContainerStatuses[0].RestartCount))
@@ -414,6 +415,23 @@ var _ = Describe("CDI ingress config tests", func() {
 			return *config.Status.UploadProxyURL
 		}, time.Second*30, time.Second).Should(Equal(override))
 	})
+
+	It("Should not crash and uploadProxy empty, if ingress defined without service", func() {
+		ingress = createNoServiceIngress("test-ingress", f.CdiInstallNs)
+		// The cdi controller will not process this ingress because it doesn't have the url we are looking for.
+		_, err := f.K8sClient.NetworkingV1().Ingresses(f.CdiInstallNs).Create(context.TODO(), ingress, metav1.CreateOptions{})
+		Expect(err).ToNot(HaveOccurred())
+		By("Expecting uploadproxy url to be the default url")
+		Eventually(func() string {
+			config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			proxyUrl := ""
+			if config.Status.UploadProxyURL != nil {
+				proxyUrl = *config.Status.UploadProxyURL
+			}
+			return proxyUrl
+		}, time.Second*30, time.Second).Should(Equal(defaultUrl))
+	})
 })
 
 var _ = Describe("CDI route config tests", func() {
@@ -474,25 +492,27 @@ var _ = Describe("CDI route config tests", func() {
 var _ = Describe("CDIConfig instance management", func() {
 	f := framework.NewFramework("cdiconfig-test")
 
-	It("[test_id:4952]Should re-create the object if deleted", func() {
-		By("Verifying the object exists")
-		config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
-		Expect(err).ToNot(HaveOccurred())
-		// Save the UID, so we can check it against a new one.
-		orgUID := config.GetUID()
-		GinkgoWriter.Write([]byte(fmt.Sprintf("Original CDIConfig UID: %s\n", orgUID)))
-		By("Deleting the object")
-		err = f.CdiClient.CdiV1beta1().CDIConfigs().Delete(context.TODO(), config.Name, metav1.DeleteOptions{})
-		Expect(err).ToNot(HaveOccurred())
+	Context("[Destructive]", func() {
+		It("[test_id:4952]Should re-create the object if deleted", func() {
+			By("Verifying the object exists")
+			config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			// Save the UID, so we can check it against a new one.
+			orgUID := config.GetUID()
+			GinkgoWriter.Write([]byte(fmt.Sprintf("Original CDIConfig UID: %s\n", orgUID)))
+			By("Deleting the object")
+			err = f.CdiClient.CdiV1beta1().CDIConfigs().Delete(context.TODO(), config.Name, metav1.DeleteOptions{})
+			Expect(err).ToNot(HaveOccurred())
 
-		Eventually(func() bool {
-			newConfig, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
-			if err != nil {
-				return false
-			}
-			GinkgoWriter.Write([]byte(fmt.Sprintf("New CDIConfig UID: %s\n", newConfig.GetUID())))
-			return orgUID != newConfig.GetUID()
-		}, time.Second*30, time.Second).Should(BeTrue())
+			Eventually(func() bool {
+				newConfig, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+				if err != nil {
+					return false
+				}
+				GinkgoWriter.Write([]byte(fmt.Sprintf("New CDIConfig UID: %s\n", newConfig.GetUID())))
+				return orgUID != newConfig.GetUID() && !apiequality.Semantic.DeepEqual(newConfig.Status, cdiv1.CDIConfigStatus{})
+			}, time.Second*30, time.Second).Should(BeTrue())
+		})
 	})
 })
 
@@ -606,6 +626,41 @@ func createIngress(name, ns, service, hostUrl string) *networkingv1.Ingress {
 	}
 }
 
+func createNoServiceIngress(name, ns string) *networkingv1.Ingress {
+	apiGroup := "test.test"
+	return &networkingv1.Ingress{
+		TypeMeta: metav1.TypeMeta{
+			APIVersion: "networking/v1",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      name,
+			Namespace: ns,
+			Labels: map[string]string{
+				"nginx.org/ssl-services":                      "cdi-uploadproxy",
+				"ingress.kubernetes.io/ssl-passthrough":       "true",
+				"nginx.ingress.kubernetes.io/secure-backends": "true",
+				"nginx.ingress.kubernetes.io/proxy-body-size": "0",
+			},
+		},
+		Spec: networkingv1.IngressSpec{
+			DefaultBackend: &networkingv1.IngressBackend{
+				Resource: &v1.TypedLocalObjectReference{
+					APIGroup: &apiGroup,
+					Kind:     "test",
+					Name:     "test",
+				},
+			},
+			TLS: []networkingv1.IngressTLS{
+				{
+					Hosts: []string{
+						"cdi-uploadproxy.example.com",
+					},
+				},
+			},
+		},
+	}
+}
+
 func SetStorageClassDefault(f *framework.Framework, scName string, isDefault bool) error {
 	sc, err := f.K8sClient.StorageV1().StorageClasses().Get(context.TODO(), scName, metav1.GetOptions{})
 	if err != nil {
