@@ -6,6 +6,7 @@ import (
 	"fmt"
 	"io/ioutil"
 	"net/http"
+	"os"
 	"os/exec"
 	"reflect"
 	"regexp"
@@ -13,6 +14,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/google/uuid"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/ginkgo/extensions/table"
 	. "github.com/onsi/gomega"
@@ -20,7 +22,10 @@ import (
 	v1 "k8s.io/api/core/v1"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/runtime/schema"
+	"k8s.io/apimachinery/pkg/types"
 
 	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
@@ -50,6 +55,7 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 	})
 
 	DescribeTable("[test_id:2329] Should fail to import images that require too much space", func(uploadURL string) {
+		startTime := time.Now()
 		imageURL := fmt.Sprintf(uploadURL, f.CdiInstallNs)
 
 		By(imageURL)
@@ -65,21 +71,17 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 		Expect(err).NotTo(HaveOccurred(), fmt.Sprintf("Unable to get importer pod"))
 
 		By(fmt.Sprintf("logs for pod -n %s %s", importer.Name, importer.Namespace))
-		By("Verify size error in logs")
+		By("Verify datavolume too small condition")
 		Eventually(func() bool {
-			log, _ := tests.RunKubectlCommand(f, "logs", importer.Name, "-n", importer.Namespace)
-			if strings.Contains(log, "is larger than available size") {
-				return true
-			}
-			if strings.Contains(log, "no space left on device") {
-				return true
-			}
-			if strings.Contains(log, "qemu-img execution failed") {
-				return true
-			}
-			By("Failed to find error messages about a too large image in log:")
-			By(log)
-			return false
+			dv, err = f.CdiClient.CdiV1beta1().DataVolumes(f.Namespace.Name).Get(context.TODO(), dv.Name, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			return tests.VerifyConditions(dv.Status.Conditions, startTime, &cdiv1.DataVolumeCondition{
+				Type:    cdiv1.DataVolumeRunning,
+				Status:  v1.ConditionFalse,
+				Message: "DataVolume too small to contain image",
+				Reason:  "Error",
+			})
 		}, controllerSkipPVCCompleteTimeout, assertionPollInterval).Should(BeTrue())
 	},
 		Entry("fail given a large virtual size RAW XZ file", utils.LargeVirtualDiskXz),
@@ -134,6 +136,106 @@ var _ = Describe("[rfe_id:1115][crit:high][vendor:cnv-qe@redhat.com][level:compo
 			Expect(f.GetDiskGroup(f.Namespace, pvc, false)).To(Equal("107"))
 		}
 	})
+
+	It("[test_id:6687] Should retain the importer pod after completion with dv annotation cdi.kubevirt.io/storage.pod.retainAfterCompletion=true", func() {
+		dataVolume := utils.NewDataVolumeWithHTTPImport("import-pod-retain-test", "100Mi", fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs))
+		By(fmt.Sprintf("Create new datavolume %s", dataVolume.Name))
+		dataVolume.Annotations[controller.AnnPodRetainAfterCompletion] = "true"
+		dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("Verify pvc was created")
+		pvc, err := utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		f.ForceBindIfWaitForFirstConsumer(pvc)
+
+		By("Wait for import to be completed")
+		err = utils.WaitForDataVolumePhase(f.CdiClient, dataVolume.Namespace, cdiv1.Succeeded, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred(), "Datavolume not in phase succeeded in time")
+
+		By("Find importer pod after completion")
+		importer, err := utils.FindPodByPrefixOnce(f.K8sClient, dataVolume.Namespace, common.ImporterPodName, common.CDILabelSelector)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(importer.DeletionTimestamp).To(BeNil())
+	})
+})
+
+var _ = Describe("[Istio] Namespace sidecar injection", func() {
+	var (
+		f = framework.NewFramework(namespacePrefix)
+
+		// Istio sidecar injection prevents access to external resources, so we cannot use internal urls (http://cdi-file-host) for the test
+		tinyCoreIsoExternalURL = "http://tinycorelinux.net/12.x/x86/release/TinyCore-current.iso"
+	)
+
+	BeforeEach(func() {
+		value := os.Getenv("KUBEVIRT_DEPLOY_ISTIO")
+		if value != "true" {
+			Skip("No Istio enabled, skipping.")
+		}
+
+		By("Enable istio sidecar injection in namespace")
+		labelPatch := `[{"op":"add","path":"/metadata/labels/istio-injection","value":"enabled" }]`
+		_, err := f.K8sClient.CoreV1().Namespaces().Patch(context.TODO(), f.Namespace.Name, types.JSONPatchType, []byte(labelPatch), metav1.PatchOptions{})
+		Expect(err).ShouldNot(HaveOccurred())
+
+		By("Create istio sidecar")
+		sidecarRes := schema.GroupVersionResource{Group: "networking.istio.io", Version: "v1beta1", Resource: "sidecars"}
+		registryOnlySidecar := generateRegistryOnlySidecar()
+		_, err = f.DynamicClient.Resource(sidecarRes).Namespace(f.Namespace.Name).Create(context.TODO(), registryOnlySidecar, metav1.CreateOptions{})
+		Expect(err).ShouldNot(HaveOccurred())
+	})
+
+	It("[test_id:6498] Should fail to import with namespace sidecar injection enabled, and sidecar.istio.io/inject set to true", func() {
+		dataVolume := utils.NewDataVolumeWithHTTPImport("istio-sidecar-injection-test", "100Mi", tinyCoreIsoExternalURL)
+		By(fmt.Sprintf("Create new datavolume %s", dataVolume.Name))
+		// We set the Immediate Binding annotation to true, to eliminate creation of the consumer pod, which will also fail due to the Istio sidecar.
+		dataVolume.Annotations[controller.AnnImmediateBinding] = "true"
+		dataVolume.Annotations[controller.AnnPodSidecarInjection] = "true"
+		dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+		Expect(err).ToNot(HaveOccurred())
+
+		var importer *v1.Pod
+		By("Find importer pod")
+		Eventually(func() bool {
+			importer, err = utils.FindPodByPrefix(f.K8sClient, dataVolume.Namespace, common.ImporterPodName, common.CDILabelSelector)
+			return err == nil
+		}, timeout, pollingInterval).Should(BeTrue())
+
+		By("Verify HTTP request error in importer log")
+		Eventually(func() bool {
+			log, _ := tests.RunKubectlCommand(f, "logs", importer.Name, "-n", importer.Namespace)
+			if strings.Contains(log, "HTTP request errored") {
+				return true
+			}
+			if strings.Contains(log, "502 Bad Gateway") {
+				return true
+			}
+			return false
+		}, time.Minute, pollingInterval).Should(BeTrue())
+	})
+
+	It("[test_id:6492] Should successfully import with namespace sidecar injection enabled and default sidecar.istio.io/inject", func() {
+		dataVolume := utils.NewDataVolumeWithHTTPImport("istio-sidecar-injection-test", "100Mi", tinyCoreIsoExternalURL)
+		By(fmt.Sprintf("Create new datavolume %s", dataVolume.Name))
+		dataVolume.Annotations[controller.AnnImmediateBinding] = "true"
+		dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("Verify pvc was created")
+		_, err = utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("Find importer pod")
+		Eventually(func() bool {
+			_, err = utils.FindPodByPrefix(f.K8sClient, dataVolume.Namespace, common.ImporterPodName, common.CDILabelSelector)
+			return err == nil
+		}, timeout, pollingInterval).Should(BeTrue())
+
+		By("Wait for import to be completed")
+		err = utils.WaitForDataVolumePhase(f.CdiClient, dataVolume.Namespace, cdiv1.Succeeded, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred(), "Datavolume not in phase succeeded in time")
+	})
 })
 
 var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", func() {
@@ -142,8 +244,8 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 	f := framework.NewFramework(namespacePrefix)
 
 	// An image that fails import
-	invalidQcowLargeSize := func() string {
-		return fmt.Sprintf(utils.InvalidQcowImagesURL+"invalid-qcow-large-size.img", f.CdiInstallNs)
+	imageTooLargeSize := func() string {
+		return fmt.Sprintf(utils.LargeVirtualDiskXz, f.CdiInstallNs)
 	}
 
 	BeforeEach(func() {
@@ -171,8 +273,8 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 		Eventually(func() bool {
 			cr, err = f.CdiClient.CdiV1beta1().CDIs().Get(context.TODO(), "cdi", metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
-			return reflect.DeepEqual(cr.Spec, oldSpec)
-		}, 30*time.Second, time.Second)
+			return reflect.DeepEqual(cr.Spec, *oldSpec)
+		}, 30*time.Second, time.Second).Should(BeTrue())
 	})
 
 	It("[test_id:4783] Should create import pod with node placement", func() {
@@ -184,9 +286,9 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 			realCR, err := f.CdiClient.CdiV1beta1().CDIs().Get(context.TODO(), "cdi", metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			return reflect.DeepEqual(cr.Spec, realCR.Spec)
-		}, 30*time.Second, time.Second)
+		}, 30*time.Second, time.Second).Should(BeTrue())
 
-		dv := utils.NewDataVolumeWithHTTPImport("node-placement-test", "100Mi", invalidQcowLargeSize())
+		dv := utils.NewDataVolumeWithHTTPImport("node-placement-test", "100Mi", imageTooLargeSize())
 		dv, err = utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dv)
 		Expect(err).ToNot(HaveOccurred())
 
@@ -204,11 +306,15 @@ var _ = Describe("[rfe_id:4784][crit:high] Importer respects node placement", fu
 })
 
 var _ = Describe("Importer CDI config manipulation tests", func() {
-	var config *cdiv1.CDIConfig
-	var origSpec *cdiv1.CDIConfigSpec
-	var err error
 	f := framework.NewFramework(namespacePrefix)
 
+	var (
+		config         *cdiv1.CDIConfig
+		origSpec       *cdiv1.CDIConfigSpec
+		err            error
+		tinyCoreIsoURL = func() string { return fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs) }
+	)
+
 	BeforeEach(func() {
 		config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
 		Expect(err).ToNot(HaveOccurred())
@@ -223,41 +329,27 @@ var _ = Describe("Importer CDI config manipulation tests", func() {
 		})
 
 		Eventually(func() bool {
-			config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), "cdi", metav1.GetOptions{})
+			config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
-			return reflect.DeepEqual(config.Spec, origSpec)
-		}, 30*time.Second, time.Second)
+			return reflect.DeepEqual(config.Spec, *origSpec)
+		}, 30*time.Second, time.Second).Should(BeTrue())
 	})
 
-	DescribeTable("Filesystem overhead is honored with a RAW file", func(expectedSuccess bool, globalOverhead, scOverhead string) {
-		defaultSCName := utils.DefaultStorageClass.GetName()
-		testedFilesystemOverhead := &cdiv1.FilesystemOverhead{}
-		if globalOverhead != "" {
-			testedFilesystemOverhead.Global = cdiv1.Percent(globalOverhead)
-		}
-		if scOverhead != "" {
-			testedFilesystemOverhead.StorageClass = map[string]cdiv1.Percent{defaultSCName: cdiv1.Percent(scOverhead)}
-		}
-		config.Spec.FilesystemOverhead = testedFilesystemOverhead.DeepCopy()
-		By(fmt.Sprintf("Updating CDIConfig filesystem overhead to %v", config.Spec.FilesystemOverhead))
-		err := utils.UpdateCDIConfig(f.CrClient, func(config *cdiv1.CDIConfigSpec) {
-			config.FilesystemOverhead = testedFilesystemOverhead.DeepCopy()
-		})
-		Expect(err).ToNot(HaveOccurred())
-		By(fmt.Sprintf("Waiting for filsystem overhead status to be set to %v", testedFilesystemOverhead))
-		Eventually(func() bool {
-			config, err := f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
-			Expect(err).ToNot(HaveOccurred())
-			if scOverhead != "" {
-				return config.Status.FilesystemOverhead.StorageClass[defaultSCName] == cdiv1.Percent(scOverhead)
-			}
-			return config.Status.FilesystemOverhead.StorageClass[defaultSCName] == cdiv1.Percent(globalOverhead)
-		}, timeout, pollingInterval).Should(BeTrue(), "CDIConfig filesystem overhead wasn't set")
+	dvWithPvcSpec := func() *cdiv1.DataVolume {
+		return utils.NewDataVolumeWithHTTPImport("too-large-import", "50Mi", tinyCoreIsoURL())
+	}
+	dvWithStorageSpec := func() *cdiv1.DataVolume {
+		return utils.NewDataVolumeWithHTTPImportAndStorageSpec("too-large-import", "50Mi", tinyCoreIsoURL())
+	}
+	// value of .80 means 80% of filesystem is reserved for metadata,
+	// given 100Mi PVC, 80Mi is reserved, 20Mi available for image, we test using 50Mi PVC and around 20Mi image
+	highFsOverhead := "0.80"
 
-		imageURL := fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs)
+	DescribeTable("Filesystem overhead is honored with a RAW file", func(dvFunc func() *cdiv1.DataVolume, expectedSuccess bool, globalOverhead, scOverhead string) {
+		tests.SetFilesystemOverhead(f, globalOverhead, scOverhead)
 
-		By(imageURL)
-		dv := utils.NewDataVolumeWithHTTPImport("too-large-import", "500Mi", imageURL)
+		dv := dvFunc()
+		By(dv.Name)
 		dv, err = utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dv)
 		Expect(err).ToNot(HaveOccurred())
 
@@ -283,19 +375,17 @@ var _ = Describe("Importer CDI config manipulation tests", func() {
 				if strings.Contains(log, "no space left on device") {
 					return true
 				}
-				if strings.Contains(log, "qemu-img execution failed") {
-					return true
-				}
 				By("Failed to find error messages about a too large image in log:")
 				By(log)
 				return false
 			}, controllerSkipPVCCompleteTimeout, assertionPollInterval).Should(BeTrue())
 		}
 	},
-		Entry("Succeed with low global overhead", true, "0.1", ""),
-		Entry("[posneg:negative] Fail with high global overhead", false, "0.99", ""),
-		Entry("Succeed with low per-storageclass overhead (despite high global overhead)", true, "0.99", "0.1"),
-		Entry("[posneg:negative] Fail with high per-storageclass overhead (despite low global overhead)", false, "0.1", "0.99"),
+		Entry("Succeed with low global overhead", dvWithPvcSpec, true, "0.1", ""),
+		Entry("[posneg:negative] Fail with high global overhead", dvWithPvcSpec, false, highFsOverhead, ""),
+		Entry("Succeed with high global overhead when using spec.storage", dvWithStorageSpec, true, highFsOverhead, ""),
+		Entry("Succeed with low per-storageclass overhead (despite high global overhead)", dvWithPvcSpec, true, highFsOverhead, "0.1"),
+		Entry("[posneg:negative] Fail with high per-storageclass overhead (despite low global overhead)", dvWithPvcSpec, false, "0.1", highFsOverhead),
 	)
 })
 
@@ -406,8 +496,11 @@ func startPrometheusPortForward(f *framework.Framework) (string, *exec.Cmd, erro
 
 var _ = Describe("Importer Test Suite-Block_device", func() {
 	f := framework.NewFramework(namespacePrefix)
-	var pvc *v1.PersistentVolumeClaim
-	var err error
+	var (
+		pvc            *v1.PersistentVolumeClaim
+		err            error
+		tinyCoreIsoURL = func() string { return fmt.Sprintf(utils.TinyCoreIsoURL, f.CdiInstallNs) }
+	)
 
 	AfterEach(func() {
 		if pvc != nil {
@@ -475,6 +568,56 @@ var _ = Describe("Importer Test Suite-Block_device", func() {
 		}, controllerSkipPVCCompleteTimeout, assertionPollInterval).Should(BeTrue())
 		Expect(err).ToNot(HaveOccurred())
 	})
+
+	It("Should perform fsync syscall after qemu-img convert to raw", func() {
+		if !f.IsBlockVolumeStorageClassAvailable() {
+			Skip("Storage Class for block volume is not available")
+		}
+		dataVolume := utils.NewDataVolumeWithHTTPImportToBlockPV("qemu-img-convert-fsync-test", "4Gi", tinyCoreIsoURL(), f.BlockSCName)
+		By(fmt.Sprintf("Create new datavolume %s", dataVolume.Name))
+		dataVolume.SetAnnotations(map[string]string{})
+		dataVolume.Annotations[controller.AnnPodRetainAfterCompletion] = "true"
+		dataVolume, err := utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dataVolume)
+		Expect(err).ToNot(HaveOccurred())
+
+		var importer *v1.Pod
+		By("Find importer pod")
+		Eventually(func() bool {
+			importer, err = utils.FindPodByPrefix(f.K8sClient, dataVolume.Namespace, common.ImporterPodName, common.CDILabelSelector)
+			return err == nil
+		}, timeout, pollingInterval).Should(BeTrue())
+
+		By("Verify fsync() syscall was made")
+		Eventually(func() bool {
+			log, err := tests.RunKubectlCommand(f, "logs", importer.Name, "-n", importer.Namespace)
+			if err != nil {
+				return false
+			}
+			for _, line := range strings.Split(strings.TrimSuffix(log, "\n"), "\n") {
+				if strings.Contains(line, fmt.Sprintf("Successfully completed fsync(%s) syscall", common.WriteBlockPath)) {
+					return true
+				}
+			}
+			return false
+		}, 3*time.Minute, pollingInterval).Should(BeTrue())
+
+		phase := cdiv1.Succeeded
+		By(fmt.Sprintf("Waiting for datavolume to match phase %s", string(phase)))
+		err = utils.WaitForDataVolumePhase(f.CdiClient, f.Namespace.Name, phase, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		zero := int64(0)
+		err = utils.DeletePodByName(f.K8sClient, fmt.Sprintf("%s-%s", common.ImporterPodName, dataVolume.Name), f.Namespace.Name, &zero)
+		Expect(err).ToNot(HaveOccurred())
+
+		By("Verify content")
+		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, utils.PersistentVolumeClaimFromDataVolume(dataVolume), utils.DefaultPvcMountPath, utils.UploadFileMD5, utils.UploadFileSize)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(same).To(BeTrue())
+
+		By("Delete DV")
+		err = utils.DeleteDataVolume(f.CdiClient, f.Namespace.Name, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+	})
 })
 
 var _ = Describe("[rfe_id:1947][crit:high][test_id:2145][vendor:cnv-qe@redhat.com][level:component]Importer Archive ContentType", func() {
@@ -580,7 +723,7 @@ var _ = Describe("Namespace with quota", func() {
 		By(fmt.Sprintf("Creating PVC with endpoint annotation %q", httpEp+"/tinyCore.iso"))
 
 		pvc, err := f.CreatePVCFromDefinition(utils.NewPVCDefinition(
-			"import-image-to-block-pvc",
+			"import-image-to-pvc",
 			"500Mi",
 			pvcAnn,
 			nil))
@@ -596,7 +739,7 @@ var _ = Describe("Namespace with quota", func() {
 		}, CompletionTimeout, assertionPollInterval).Should(BeEquivalentTo(v1.PodSucceeded))
 
 		By("Verify content")
-		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, "/pvc", "d41d8cd98f00b204e9800998ecf8427e", utils.UploadFileSize)
+		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, utils.DefaultImagePath, utils.UploadFileMD5, utils.UploadFileSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(same).To(BeTrue())
 		By("Verifying permissions are 660")
@@ -673,7 +816,7 @@ var _ = Describe("Namespace with quota", func() {
 		}, CompletionTimeout, assertionPollInterval).Should(BeEquivalentTo(v1.PodSucceeded))
 
 		By("Verify content")
-		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, "/pvc", "d41d8cd98f00b204e9800998ecf8427e", utils.UploadFileSize)
+		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, utils.DefaultImagePath, utils.UploadFileMD5, utils.UploadFileSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(same).To(BeTrue())
 		By("Verifying permissions are 660")
@@ -709,7 +852,7 @@ var _ = Describe("Namespace with quota", func() {
 		}, CompletionTimeout, assertionPollInterval).Should(BeEquivalentTo(v1.PodSucceeded))
 
 		By("Verify content")
-		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, "/pvc", "d41d8cd98f00b204e9800998ecf8427e", utils.UploadFileSize)
+		same, err := f.VerifyTargetPVCContentMD5(f.Namespace, pvc, utils.DefaultImagePath, utils.UploadFileMD5, utils.UploadFileSize)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(same).To(BeTrue())
 		By("Verifying permissions are 660")
@@ -938,8 +1081,19 @@ var _ = Describe("Preallocation", func() {
 		tinyCoreTarURL      = func() string { return fmt.Sprintf(utils.TarArchiveURL, f.CdiInstallNs) }
 		tinyCoreRegistryURL = func() string { return fmt.Sprintf(utils.TinyCoreIsoRegistryURL, f.CdiInstallNs) }
 		imageioURL          = func() string { return fmt.Sprintf(utils.ImageioURL, f.CdiInstallNs) }
+		vcenterURL          = func() string { return fmt.Sprintf(utils.VcenterURL, f.CdiInstallNs) }
+		md5PrefixSize       = int64(100000)
+		config              *cdiv1.CDIConfig
+		origSpec            *cdiv1.CDIConfigSpec
 	)
 
+	BeforeEach(func() {
+		config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+		Expect(err).ToNot(HaveOccurred())
+
+		origSpec = config.Spec.DeepCopy()
+	})
+
 	AfterEach(func() {
 		By("Delete DV")
 		err := utils.DeleteDataVolume(f.CdiClient, f.Namespace.Name, dataVolume.Name)
@@ -952,6 +1106,17 @@ var _ = Describe("Preallocation", func() {
 			}
 			return false
 		}, timeout, pollingInterval).Should(BeTrue())
+
+		By("Restoring CDIConfig to original state")
+		err = utils.UpdateCDIConfig(f.CrClient, func(config *cdiv1.CDIConfigSpec) {
+			origSpec.DeepCopyInto(config)
+		})
+
+		Eventually(func() bool {
+			config, err = f.CdiClient.CdiV1beta1().CDIConfigs().Get(context.TODO(), common.ConfigName, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			return reflect.DeepEqual(config.Spec, *origSpec)
+		}, 30*time.Second, time.Second).Should(BeTrue())
 	})
 
 	It("Importer should add preallocation when requested", func() {
@@ -973,7 +1138,16 @@ var _ = Describe("Preallocation", func() {
 
 		pvc, err = utils.FindPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).Should(Or(Equal("true"), Equal("skipped")))
+		Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).Should(Equal("true"))
+
+		By("Verify content")
+		md5, err := f.GetMD5(f.Namespace, pvc, utils.DefaultImagePath, md5PrefixSize)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(md5).To(Equal(utils.TinyCoreMD5))
+
+		ok, err := f.VerifyImagePreallocated(f.Namespace, pvc)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(ok).To(BeTrue())
 	})
 
 	It("Importer should not add preallocation when preallocation=false", func() {
@@ -996,10 +1170,20 @@ var _ = Describe("Preallocation", func() {
 
 		pvc, err = utils.FindPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
 		Expect(err).ToNot(HaveOccurred())
-		Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).ShouldNot(Or(Equal("true"), Equal("skipped")))
+		Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).ShouldNot(Equal("true"))
+
+		By("Verify content")
+		md5, err := f.GetMD5(f.Namespace, pvc, utils.DefaultImagePath, md5PrefixSize)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(md5).To(Equal(utils.TinyCoreMD5))
+
+		By("Verify preallocated size")
+		ok, err := f.VerifyImagePreallocated(f.Namespace, pvc)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(ok).To(BeFalse())
 	})
 
-	DescribeTable("All import paths should contain Preallocation step", func(shouldPreallocate bool, dvFunc func() *cdiv1.DataVolume) {
+	DescribeTable("All import paths should contain Preallocation step", func(shouldPreallocate bool, expectedMD5, path string, dvFunc func() *cdiv1.DataVolume) {
 		dv := dvFunc()
 		By(fmt.Sprintf("Creating new datavolume %s", dv.Name))
 		preallocation := true
@@ -1030,45 +1214,57 @@ var _ = Describe("Preallocation", func() {
 		pvc, err = utils.FindPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
 		Expect(err).ToNot(HaveOccurred())
 		if shouldPreallocate {
-			Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).Should(Or(Equal("true"), Equal("skipped")))
+			Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).Should(Equal("true"))
+
+			By("Verify content")
+			md5, err := f.GetMD5(f.Namespace, pvc, path, md5PrefixSize)
+			Expect(err).ToNot(HaveOccurred())
+			Expect(md5).To(Equal(expectedMD5))
+
+			if !f.IsBlockVolumeStorageClassAvailable() {
+				// Block volumes can't be read with qemu-img
+				ok, err := f.VerifyImagePreallocated(f.Namespace, pvc)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(ok).To(BeTrue())
+			}
 		} else {
-			Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).ShouldNot(Or(Equal("true"), Equal("skipped")))
+			Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).ShouldNot(Equal("true"))
 		}
 	},
-		Entry("HTTP import (ISO image)", true, func() *cdiv1.DataVolume {
+		Entry("HTTP import (ISO image)", true, utils.TinyCoreMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeWithHTTPImport("import-dv", "100Mi", tinyCoreIsoURL())
 		}),
-		Entry("HTTP import (QCOW2 image)", true, func() *cdiv1.DataVolume {
+		Entry("HTTP import (QCOW2 image)", true, utils.TinyCoreMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeWithHTTPImport("import-dv", "100Mi", tinyCoreQcow2URL())
 		}),
-		Entry("HTTP import (TAR image)", true, func() *cdiv1.DataVolume {
+		Entry("HTTP import (TAR image)", true, utils.TinyCoreTarMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeWithHTTPImport("import-dv", "100Mi", tinyCoreTarURL())
 		}),
-		Entry("HTTP import (archive content)", false, func() *cdiv1.DataVolume {
+		Entry("HTTP import (archive content)", false, "", "", func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeWithArchiveContent("import-dv", "100Mi", tinyCoreTarURL())
 		}),
-		Entry("HTTP Import (TAR image, block DataVolume)", true, func() *cdiv1.DataVolume {
+		Entry("HTTP Import (TAR image, block DataVolume)", true, utils.TinyCoreTarMD5, utils.DefaultPvcMountPath, func() *cdiv1.DataVolume {
 			if !f.IsBlockVolumeStorageClassAvailable() {
 				Skip("Storage Class for block volume is not available")
 			}
 
 			return utils.NewDataVolumeWithHTTPImportToBlockPV("import-dv", "4Gi", tinyCoreTarURL(), f.BlockSCName)
 		}),
-		Entry("HTTP Import (ISO image, block DataVolume)", true, func() *cdiv1.DataVolume {
+		Entry("HTTP Import (ISO image, block DataVolume)", true, utils.TinyCoreMD5, utils.DefaultPvcMountPath, func() *cdiv1.DataVolume {
 			if !f.IsBlockVolumeStorageClassAvailable() {
 				Skip("Storage Class for block volume is not available")
 			}
 
 			return utils.NewDataVolumeWithHTTPImportToBlockPV("import-dv", "4Gi", tinyCoreIsoURL(), f.BlockSCName)
 		}),
-		Entry("HTTP Import (QCOW2 image, block DataVolume)", true, func() *cdiv1.DataVolume {
+		Entry("HTTP Import (QCOW2 image, block DataVolume)", true, utils.TinyCoreMD5, utils.DefaultPvcMountPath, func() *cdiv1.DataVolume {
 			if !f.IsBlockVolumeStorageClassAvailable() {
 				Skip("Storage Class for block volume is not available")
 			}
 
 			return utils.NewDataVolumeWithHTTPImportToBlockPV("import-dv", "4Gi", tinyCoreQcow2URL(), f.BlockSCName)
 		}),
-		Entry("ImageIO import", true, func() *cdiv1.DataVolume {
+		Entry("ImageIO import", true, utils.ImageioMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			cm, err := utils.CopyImageIOCertConfigMap(f.K8sClient, f.Namespace.Name, f.CdiInstallNs)
 			Expect(err).To(BeNil())
 			stringData := map[string]string{
@@ -1078,22 +1274,103 @@ var _ = Describe("Preallocation", func() {
 			s, _ := utils.CreateSecretFromDefinition(f.K8sClient, utils.NewSecretDefinition(nil, stringData, nil, f.Namespace.Name, "mysecret"))
 			return utils.NewDataVolumeWithImageioImport("import-dv", "100Mi", imageioURL(), s.Name, cm, "123")
 		}),
-		Entry("Registry import", true, func() *cdiv1.DataVolume {
+		Entry("Registry import", true, utils.TinyCoreMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			dataVolume = utils.NewDataVolumeWithRegistryImport("import-dv", "100Mi", tinyCoreRegistryURL())
 			cm, err := utils.CopyRegistryCertConfigMap(f.K8sClient, f.Namespace.Name, f.CdiInstallNs)
 			Expect(err).To(BeNil())
 			dataVolume.Spec.Source.Registry.CertConfigMap = cm
 			return dataVolume
 		}),
-		Entry("Blank image", true, func() *cdiv1.DataVolume {
+		Entry("VddkImport", true, utils.VcenterMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
+			// Find vcenter-simulator pod
+			pod, err := utils.FindPodByPrefix(f.K8sClient, f.CdiInstallNs, "vcenter-deployment", "app=vcenter")
+			Expect(err).ToNot(HaveOccurred())
+			Expect(pod).ToNot(BeNil())
+
+			// Get test VM UUID
+			id, err := tests.RunKubectlCommand(f, "exec", "-n", pod.Namespace, pod.Name, "--", "cat", "/tmp/vmid")
+			Expect(err).To(BeNil())
+			vmid, err := uuid.Parse(strings.TrimSpace(id))
+			Expect(err).To(BeNil())
+
+			// Get disk name
+			disk, err := tests.RunKubectlCommand(f, "exec", "-n", pod.Namespace, pod.Name, "--", "cat", "/tmp/vmdisk")
+			Expect(err).To(BeNil())
+			disk = strings.TrimSpace(disk)
+			Expect(err).To(BeNil())
+
+			// Create VDDK login secret
+			stringData := map[string]string{
+				common.KeyAccess: "user",
+				common.KeySecret: "pass",
+			}
+			backingFile := disk
+			secretRef := "vddksecret"
+			thumbprint := "testprint"
+			s, _ := utils.CreateSecretFromDefinition(f.K8sClient, utils.NewSecretDefinition(nil, stringData, nil, f.Namespace.Name, secretRef))
+
+			return utils.NewDataVolumeWithVddkImport("import-dv", "100Mi", backingFile, s.Name, thumbprint, vcenterURL(), vmid.String())
+		}),
+		Entry("Blank image", true, utils.BlankMD5, utils.DefaultImagePath, func() *cdiv1.DataVolume {
 			return utils.NewDataVolumeForBlankRawImage("import-dv", "100Mi")
 		}),
-		Entry("Blank block DataVolume", true, func() *cdiv1.DataVolume {
+		Entry("Blank block DataVolume", true, utils.BlankMD5, "/pvc", func() *cdiv1.DataVolume {
 			if !f.IsBlockVolumeStorageClassAvailable() {
 				Skip("Storage Class for block volume is not available")
 			}
 
-			return utils.NewDataVolumeForBlankRawImageBlock("import-dv", "100Mi", f.BlockSCName)
+			return utils.NewDataVolumeForBlankRawImageBlock("import-dv", "1Gi", f.BlockSCName)
 		}),
 	)
+
+	It("Filesystem overhead is honored with blank volume", func() {
+		tests.SetFilesystemOverhead(f, "0.055", "0.055")
+
+		dv := utils.NewDataVolumeForBlankRawImage("import-dv", "100Mi")
+		preallocation := true
+		dv.Spec.Preallocation = &preallocation
+		dataVolume, err = utils.CreateDataVolumeFromDefinition(f.CdiClient, f.Namespace.Name, dv)
+		Expect(err).ToNot(HaveOccurred())
+
+		pvc, err := utils.WaitForPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		f.ForceBindIfWaitForFirstConsumer(pvc)
+
+		By("Verify the pod status is succeeded on the target PVC")
+		found, err := utils.WaitPVCPodStatusSucceeded(f.K8sClient, pvc)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(found).To(BeTrue())
+
+		Expect(f.VerifyFSOverhead(f.Namespace, pvc, preallocation)).To(BeTrue())
+
+		pvc, err = utils.FindPVC(f.K8sClient, dataVolume.Namespace, dataVolume.Name)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(pvc.GetAnnotations()[controller.AnnPreallocationApplied]).Should(Equal("true"))
+
+		By("Verify content")
+		md5, err := f.GetMD5(f.Namespace, pvc, utils.DefaultImagePath, md5PrefixSize)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(md5).To(Equal(utils.BlankMD5))
+
+		ok, err := f.VerifyImagePreallocated(f.Namespace, pvc)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(ok).To(BeTrue())
+	})
 })
+
+func generateRegistryOnlySidecar() *unstructured.Unstructured {
+	return &unstructured.Unstructured{
+		Object: map[string]interface{}{
+			"apiVersion": "networking.istio.io/v1beta1",
+			"kind":       "Sidecar",
+			"metadata": map[string]interface{}{
+				"name": "registry-only-sidecar",
+			},
+			"spec": map[string]interface{}{
+				"outboundTrafficPolicy": map[string]interface{}{
+					"mode": "REGISTRY_ONLY",
+				},
+			},
+		},
+	}
+}
